package com.unissoft.filter;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.unissoft.constant.ContentType;
import com.unissoft.constant.OrderedConstant;
import com.unissoft.model.LogHelper;
import com.unissoft.model.SysLog;
import com.unissoft.model.SysUserPO;
import com.unissoft.utils.GenerateIdUtils;
import com.unissoft.utils.IpUtils;
import com.unissoft.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.security.jwt.Jwt;
import org.springframework.security.jwt.JwtHelper;
import org.springframework.stereotype.Component;
import org.springframework.util.StopWatch;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.net.URI;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

/**
 * @author tongxi
 * 请求日志打印
 */
@Component
@Slf4j
public class RequestLogFilter implements GlobalFilter, Ordered {

    @Autowired
    private RedisUtil redisUtil;

    @Override
    public int getOrder() {
        return OrderedConstant.REQUEST_FILTER;
    }

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        try {
            ServerHttpRequest request = exchange.getRequest();
            // 设置X-Request-Id
            AtomicReference<String> requestId = new AtomicReference<>(GenerateIdUtils.requestIdWithUUID());
            URI requestUri = request.getURI();
            String uriQuery = requestUri.getQuery();
            String url = requestUri.getPath() + (StringUtils.isNotBlank(uriQuery) ? "?" + uriQuery : "");
            HttpHeaders headers = request.getHeaders();
            MediaType mediaType = headers.getContentType();
            String method = request.getMethodValue().toUpperCase();

            // 原始请求体
            final AtomicReference<String> requestBody = new AtomicReference<>();
            final AtomicBoolean newBody = new AtomicBoolean(false);
            if (Objects.nonNull(mediaType) && LogHelper.isUploadFile(mediaType)) {
                requestBody.set(ContentType.UPLOAD_FILE);
            } else {
                if (method.equals(ContentType.GET)) {
                    if (StringUtils.isNotBlank(uriQuery)) {
                        requestBody.set(uriQuery);
                    }
                } else {
                    newBody.set(true);
                }
            }

            //获取当前操作用户名字
            String userName = "admin";
            Integer depotId = null;
            if (request.getHeaders().get(ContentType.AUTHORIZATION) != null) {
                String authorization = request.getHeaders().get(ContentType.AUTHORIZATION).get(0).replace("bearer ", "");

                Jwt decode = JwtHelper.decode(authorization);
                String claims = decode.getClaims();
                JSONObject jsonObject = JSON.parseObject(claims);
                Map<String, String> jsonMap = JSONObject.toJavaObject(jsonObject, Map.class);
                JSONObject user = JSON.parseObject(jsonMap.get("user_name"));
                SysUserPO sysUserPO = JSONObject.toJavaObject(user, SysUserPO.class);
                userName = sysUserPO.getPersonnelName();
                depotId = sysUserPO.getDepotId();
            }

            SysLog logDTO = new SysLog(url, requestBody.get(), requestId.get(), method, userName, new Date(), IpUtils.getClientIp(request), ContentType.INFO, depotId);
            //存入redis
            List<Object> logs = Lists.newArrayList(JSON.toJSONString(logDTO));

            redisUtil.lSetList(ContentType.REDIS_LOGS, logs);

            return chain.filter(exchange).then(LogHelper.doRecord(logDTO));
        } catch (Exception e) {
            log.error("请求日志打印出现异常", e);
            return chain.filter(exchange);
        }
    }

}
